#include "ClientSocket.h"
#include "SocketException.h"
#include <iostream>
#include <string>
#define TRACEPOINT_DEFINE
#include "trace.h"
#include "my_signal.h"
#include <stdlib.h>
#include <stdio.h>

void usage();

int main ( int argc, char** argv )
{
    ENTER("main");

    std::string name;

    int nbExchanges = 100;

    if ((argc < 2) || (argc > 3)) 
    {
        printf("Invalid number of arguments\n");
        usage();
        RETURN(-1);
    }
    else 
    {
        name = argv[1];
        INFO(STR("Client %s", name.c_str()));
    }
    
   name = argv[1];
   INFO(STR("Client %s", name.c_str()));
   
   if (argc == 3) 
   {
       nbExchanges = atoi (argv[2]);
       if (nbExchanges < 0) 
       {
           printf("Invalid count, %d is out of range (count >= 0) \n", nbExchanges);
           usage();
           RETURN (-1);
       }
   }
int cnt = 0; 
    printf("nbExchanges %d\n", nbExchanges);
    try
    {

        INFO(STR("host %s port %d", "localhost", 30000));

        std::string serverName = "Master";

        ClientSocket client_socket ( "localhost", 30000 );
        
        Signal signal = Signal(REGISTER_PLAYER_REQUEST, sizeof(RegisterPlayerRequest) + name.size());
        strcpy(signal.getSignal()->registerPlayerRequest.player, name.c_str()); 

	       
        TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), STR("player %s", name.c_str()));
        client_socket << signal;
	cnt++;
        bool isFinished = false;
        
        int nbRuns = 0;

        
        bool isAfterResume = false;
        
        while ( !isFinished )
        {
            Signal data;
            client_socket >> data;
            
            switch (data.getSignal()->sigNo) {
                case REGISTER_PLAYER_REPLY: 
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->registerPlayerReply.result));
                    
                    // Send game request
                    signal = Signal(GAME_REQUEST, sizeof(GameRequest));
                    signal.getSignal()->gameRequest.game = PLAY_BALL;
                    TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), STR("game %d", signal.getSignal()->gameRequest.game));
                    cnt++; 
                    client_socket << signal;
                    break;
                }
                    
                case GAME_REPLY: 
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->gameReply.result));
                    
                    // Start game request
                    signal = Signal(START_GAME_REQUEST, sizeof(StartGameRequest));
                    TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), "");
                    cnt++; 
                    client_socket << signal;
                    
                    break;
                }

                case START_GAME_REPLY:
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->startGameReply.result));
                    
                    // Send ball
                    signal = Signal(BALL_REQUEST, sizeof(BallRequest));
                    signal.getSignal()->ballRequest.count = nbRuns;
                    TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), STR("count %d", nbRuns));
                    cnt++; 
                    ++nbRuns;
                    client_socket << signal;
                    break;
                }
               
                case BALL_REPLY: 
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->ballReply.result));
                    if (nbRuns < nbExchanges) 
                    {
                        // Send ball
                        signal = Signal(BALL_REQUEST, sizeof(BallRequest));
                        signal.getSignal()->ballRequest.count = nbRuns;
                        TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), STR("count %d", nbRuns));
                    cnt++; 
                        ++nbRuns;
                    }
                    else 
                    {
                        if (isAfterResume) 
                        {
                            // Stop game request
                            signal = Signal(STOP_GAME_REQUEST, sizeof(StopGameRequest));
                            TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), "");
                    cnt++; 
                            client_socket << signal;
                        }
                        else 
                        {
                            // Pause game request
                            signal = Signal(PAUSE_GAME_REQUEST, sizeof(PauseGameRequest));
                            TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), "");
                    cnt++; 
                            isAfterResume = true;
                            nbRuns = 0;
                        }
                    }
                    client_socket << signal;
                    
                    break;
                }    

                case PAUSE_GAME_REPLY:
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->pauseGameReply.result));
                    
                    // Resume game request
                    signal = Signal(RESUME_GAME_REQUEST, sizeof(ResumeGameRequest));
                    TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), "");
                    cnt++; 
                    client_socket << signal;

                    break;
                }

                case RESUME_GAME_REPLY:
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->resumeGameReply.result));
                    
                    // Send ball
                    signal = Signal(BALL_REQUEST, sizeof(BallRequest));
                    signal.getSignal()->ballRequest.count = nbRuns;
                    TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), STR("count %d", nbRuns));
                    cnt++; 
                    ++nbRuns;
                    client_socket << signal;
                    
                    break;
                }

                case STOP_GAME_REPLY:
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->stopGameReply.result));
                    Signal signal = Signal(DEREGISTER_PLAYER_REQUEST, sizeof(DeregisterPlayerRequest) + name.size());
                    signal.getSignal()->deregisterPlayerRequest.player;
                    strcpy(signal.getSignal()->deregisterPlayerRequest.player, name.c_str()); 
                    
                    TRACE_SEND_SIG(name.c_str(), serverName.c_str(), signal.getSignalNumber(), STR("player %s", name.c_str()));
                    cnt++; 
                    client_socket << signal;

                    break;
                }

                case DEREGISTER_PLAYER_REPLY:
                {
                    TRACE_REC_SIG(serverName.c_str(), name.c_str(), data.getSignalNumber(), STR("result %d", data.getSignal()->deregisterPlayerReply.result));
                    
                    isFinished = true;
                    break;
                }   
                default:
                    TRACE_ERROR(STR("Unknown signal %d", data.getSignalNumber()));                    
                    break;
            }
        }
    }
    catch ( SocketException& e )
    {
        TRACE_ERROR(STR("Exception was caught %s", e.description().c_str()));
    }

    printf("Number of messages exchanged: %d\n", cnt);
    RETURN 0;
}

void usage() 
{
    printf("\nUsage:\n");
    printf("simple_client <player name> [<nbExchanges>]\n");
    printf("\twhere:\t<player name> is name of the player\n");
    printf("\t\t<nbExhanges> is the number of exhanged ball signals. Optional, default 100\n");
}
