##############################################################################
#
# Usage:
#     make [all | <blank>]    
#     make clean
#
# Where targets are:
#     all or <blank> - The default build.  
#     clean          - Clean the directory
##############################################################################
OBJDIR = .obj
LD_FLAGS = -g
CFLAGS = -O2
LIB = -ldl -llttng-ust-fork
CPP = g++
CC = gcc

include sources.mk

TYPE = $(shell grep type sources.mk | awk -F ":=" '{print $$2}')
MAKEFILES_DIR = ../makefiles

# include files
INC += $(addprefix -I,$(inc))

# source code directory
SRCDIR = ./

# souce code files
CFILES = $(foreach dir,$(SRCDIR),$(notdir $(wildcard $(dir)/*.cpp)))
CFILES2 = $(foreach dir,$(SRCDIR),$(notdir $(wildcard $(dir)/*.c)))

OBJS1 = $(CFILES:.cpp=.o)
OBJS2 = $(CFILES2:.c=.o)
OBJS =  $(OBJS2) $(OBJS1)

DEPS1 = $(CFILES:.cpp=.P)
DEPS2 = $(CFILES2:.c=.P)
DEPS =  $(DEPS2) $(DEPS1)

OBJS := $(addprefix $(OBJDIR)/,$(OBJS)) 

# define object files
OBJFILES = $(addsuffix /${OBJDIR}/*.o,${src})
OBJDIRS = $(addsuffix /${OBJDIR},${src})

all: 
	@if [ $(TYPE) ] ; \
		then \
		if [ $(TYPE) = "obj" ] ; \
			then \
			echo "Compiling obj ..." ; \
				$(MAKE) obj ; \
		elif [ $(TYPE) = "lib" ] ; \
			then \
				echo "Compiling lib ..." ; \
				$(MAKE) lib ; \
		else \
			echo "Compiling bin ..." ; \
			$(MAKE) bin ; \
		fi ; \
	fi

clean: 
	@if [ $(TYPE) ] ; \
		then \
		if [ $(TYPE) = "obj" ] ; \
			then \
			echo "Cleaning obj ..." ; \
				$(MAKE) objclean ; \
		elif [ $(TYPE) = "lib" ] ; \
				then \
				echo "Cleaning lib ..." ; \
				$(MAKE) libclean; \
		else \
			echo "Cleaning bin ..." ; \
			$(MAKE) binclean; \
		fi ; \
	fi

obj: $(OBJS)
	# pull in dependency info for *existing* .o files
-include $(OBJS:%.o=%.P)

$(OBJDIR)/%.o : %.cpp
	@echo ${INC}
	[ -d ${OBJDIR} ] || mkdir ${OBJDIR}
	$(CPP) ${CFLAGS} ${INC} -MMD -MF ${OBJDIR}/$*.d -c -o $@ $< 
	@cp ${OBJDIR}/$*.d ${OBJDIR}/$*.P; \
		sed -e 's/#.*//' -e 's/^[^:]*: *//' -e 's/ *\\$$//' \
			-e '/^$$/ d' -e 's/$$/ :/' < $(OBJDIR)/$*.d >> $(OBJDIR)/$*.P; \
			\rm -f $(OBJDIR)/$*.d

$(OBJDIR)/%.o : %.c
	@echo ${INC}
	[ -d ${OBJDIR} ] || mkdir ${OBJDIR}
	$(CC) ${CFLAGS} ${INC} -MMD -MF ${OBJDIR}/$*.d -c -o $@ $< 
	@cp ${OBJDIR}/$*.d ${OBJDIR}/$*.P; \
		sed -e 's/#.*//' -e 's/^[^:]*: *//' -e 's/ *\\$$//' \
			-e '/^$$/ d' -e 's/$$/ :/' < $(OBJDIR)/$*.d >> $(OBJDIR)/$*.P; \
			\rm -f $(OBJDIR)/$*.d


.PHONY: clean objclean libclean objclean

#	gcc -O2 -I. -MMD -MF .obj/tp.d -c -o .obj/tp.o tp.c
objclean :
	\rm -rf $(OBJDIR)


bin: ${name} 

${OBJDIRS}: force_lock
	[ -d $@ ] || mkdir $@     
	$(MAKE) -C $@/.. obj

${name} : ${OBJDIRS}
	$(CPP) -o $@ ${OBJFILES} ${LDFLAGS} ${LIB}

binclean :
	for dir in ${src}; do \
		$(MAKE) -C $$dir objclean; \
	done
	\rm -f ${name}

force_lock:
	true
