#ifndef TRACE_H_
#define TRACE_H_

#ifdef __cplusplus
extern "C" {
#endif

extern char* FormatString(const char *ap_format, ...);
const int MAX_FORMAT_BUFFER_LENGTH = 1024;

#include "tp.h"

#include <sys/types.h>

//**********************************************************************
// CONSTANTS AND TYPES DECLARATIONS
//**********************************************************************

#define ENTER(MSG)    /* Content of trace message           */\
        tracepoint(ust_sequence, ENTER, const_cast<char*>(__FILE__), __LINE__, const_cast<char*>(MSG))

#define RETURN \
        tracepoint(ust_sequence, RETURN, const_cast<char*>(__FILE__),__LINE__); \
        return

#define INFO(MSG)    /* Content of trace message           */\
        tracepoint(ust_sequence, INFO, const_cast<char*>(__FILE__), __LINE__, const_cast<char*>(MSG))

#define TRACE_ERROR(MSG)    /* Content of trace message           */\
        tracepoint(ust_sequence, ERROR, const_cast<char*>(__FILE__), __LINE__, const_cast<char*>(MSG))


#define TRACE(GROUP,  /* Which trace group (1 - 9) this trace belongs to */\
              MSG)    /* Content of trace message                        */\
	tracepoint(ust_sequence, TRACE##GROUP, const_cast<char*>(__FILE__), __LINE__, const_cast<char*>(MSG))

#define TRACE_SEND_SIG(SENDER,   /* Sender of signal                   */\
                       RECEIVER, /* Receiver of signal                 */\
                       SIGNAL,   /* Signal name                        */\
                       MSG)      /* Content of trace message           */\
	tracepoint(ust_sequence, SEND, const_cast<char*>(__FILE__), __LINE__, const_cast<char*>(SENDER), const_cast<char*>(RECEIVER), SIGNAL, const_cast<char*>(MSG))\


#define TRACE_REC_SIG(SENDER,   /* Sender of signal                    */\
                      RECEIVER, /* Receiver of signal                  */\
                      SIGNAL,   /* Signal name                         */\
                      MSG)      /* Content of trace message            */\
	tracepoint(ust_sequence, RECEIVE, const_cast<char*>(__FILE__), __LINE__, const_cast<char*>(SENDER), const_cast<char*>(RECEIVER), SIGNAL, const_cast<char*>(MSG))

#define TRACE_BEFORE_FORK tracepoint(ust_sequence, before_fork);\
        /* Sleep here to make sure the consumer is initialized before we fork */ \
        sleep(1)

#define TRACE_AFTER_FORK_CHILD tracepoint(ust_sequence, after_fork_child, getpid())
#define TRACE_AFTER_FORK_PARENT tracepoint(ust_sequence, after_fork_parent)

#define STR FormatString

#ifdef __cplusplus
 }

#endif
#endif /* TRACE_H_ */
