/*
 * Signal.cpp
 *
 *  Created on: Jun 6, 2011
 *      Author: "bhufmann"
 */

#include "Signal.h"

union SIGNAL {
    SIGNAL_NUMBER sigNo;
};

Signal::Signal() 
 : mp_signal(0),
   m_size(0)
{   
}

Signal::Signal(SIGNAL_NUMBER sigNo, size_t size)
{
    mp_signal = new char[size];
    (reinterpret_cast<SIGNAL*>(mp_signal))->sigNo = sigNo;
    m_size = size;
}

Signal::Signal(char* bytes, size_t size) 
{
    mp_signal = new char[size];
    memcpy(mp_signal, &bytes[0], size);
    m_size = size;
}

Signal::Signal(const Signal& other) 
{
    mp_signal = new char[other.m_size];
    memcpy(mp_signal, other.mp_signal, other.m_size);
    m_size = other.m_size;
}

Signal &Signal::operator= (const Signal & other) 
{
    if (this != &other) // protect against invalid self-assignment
    {
        // 1: allocate new memory and copy the elements
        char* newSignal = new char[other.m_size];
        memcpy(newSignal, other.mp_signal, other.m_size);

        // 2: deallocate old memory
        if (mp_signal) 
        {
            delete[] mp_signal;
        }

        // 3: assign the new memory to the object
        mp_signal = newSignal;
        m_size = other.m_size;
    }
    // by convention, always return *this
    return *this;
}

Signal::~Signal()
{
    if (mp_signal) delete mp_signal;
    mp_signal = 0;
    m_size = 0;
}

SIGNAL_NUMBER Signal::getSignalNumber() 
{
    if(mp_signal) 
    {
        return getSignal()->sigNo;
    }
    return -1;
}
