#include "ServerSocket.h"
#include "SocketException.h"
#include <string>
#define TRACEPOINT_DEFINE
#include "trace.h"
#include "my_signal.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include <stdio.h>

void messageHandler(ServerSocket *server);
void sigchld_handler(int s);
void *get_in_addr(struct sockaddr *sa);

int main ( int argc, char ** argv )
{
    ENTER("main");

    ServerSocket *new_sock = 0;

    try
    {
        // Create the socket
        INFO(STR("host %s port %d", "localhost", 30000));

        ServerSocket *server = new ServerSocket( 30000 );

        // Create Signal Handler
        struct sigaction sa;
        sa.sa_handler = sigchld_handler; // reap all dead processes
        sigemptyset(&sa.sa_mask);
        sa.sa_flags = SA_RESTART;
        if (sigaction(SIGCHLD, &sa, NULL) == -1) 
        {
            perror("sigaction");
            exit(1);
        }

        struct sockaddr_storage their_addr;

        while(1) 
        {  
            // main accept() loop
            socklen_t sin_size = sizeof their_addr;

            new_sock = new ServerSocket();
            server->accept ( *new_sock );                

            char s[INET6_ADDRSTRLEN];
            inet_ntop(their_addr.ss_family,
                    get_in_addr((struct sockaddr *)&their_addr),
                    s, sizeof s);

            printf("server: got client connection\n");

            TRACE_BEFORE_FORK;

            if (!fork()) 
            { 
                TRACE_AFTER_FORK_CHILD;

                // this is the child process
                delete server; // child doesn't need the listener

                messageHandler(new_sock);

                delete new_sock;
                RETURN 0;
            }
            TRACE_AFTER_FORK_PARENT;
            delete new_sock;  // parent doesn't need this
            new_sock = 0;
        }
    }
    catch ( SocketException& e )
    {
        TRACE_ERROR(STR("Exception caught %s\nExiting.\n", e.description().c_str()));
        if (new_sock)
            delete new_sock;
    }

    RETURN 0;
}

void messageHandler(ServerSocket *new_sock) 
{
    ENTER("messageHandler()");

    std::string serverName = "Master";
    std::string clientName;
            
    bool isFinished = false;

    while (!isFinished)
    {
        Signal data;
        *new_sock >> data;

        switch (data.getSignalNumber()) {
            case REGISTER_PLAYER_REQUEST: 
            {
                clientName = data.getSignal()->registerPlayerRequest.player;
		INFO(STR("player %s", data.getSignal()->registerPlayerRequest.player));

                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), STR("player %s", data.getSignal()->registerPlayerRequest.player));

                Signal signal = Signal(REGISTER_PLAYER_REPLY, sizeof(RegisterPlayerReply));
                signal.getSignal()->registerPlayerReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->registerPlayerReply.result));
//		TRACE_SEND_SIG2(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->registerPlayerReply.result));
                *new_sock << signal;
            }
            break;

            case GAME_REQUEST: 
            {
                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), STR("game %d", data.getSignal()->gameRequest.game));
                // Create Reply
                Signal signal = Signal(GAME_REPLY, sizeof(GameReply));
                signal.getSignal()->gameReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->gameReply.result));
                *new_sock << signal;

                break;
            }

            case START_GAME_REQUEST:
            {
                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), "");

                Signal signal = Signal(START_GAME_REPLY, sizeof(StartGameReply));
                signal.getSignal()->startGameReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->startGameReply.result));
                *new_sock << signal;

                break;
            }

            case BALL_REQUEST:
            {
                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), STR("count %d", data.getSignal()->ballRequest.count));

                // Create Reply
                Signal signal = Signal(BALL_REPLY, sizeof(BallReply));
                signal.getSignal()->ballReply.count = data.getSignal()->ballRequest.count;
                signal.getSignal()->ballReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d count %d", signal.getSignal()->ballReply.result, signal.getSignal()->ballReply.count));
                *new_sock << signal;

                break;
            }

            case PAUSE_GAME_REQUEST:
            {
                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), "");

                // Create Reply
                Signal signal = Signal(PAUSE_GAME_REPLY, sizeof(PauseGameReply));
                signal.getSignal()->pauseGameReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->pauseGameReply.result));
                *new_sock << signal;

                break;
            }

            case RESUME_GAME_REQUEST:
            {
                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), "");

                // Create Reply
                Signal signal = Signal(RESUME_GAME_REPLY, sizeof(ResumeGameReply));
                signal.getSignal()->resumeGameReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->resumeGameReply.result));
                *new_sock << signal;

                break;
            }

            case STOP_GAME_REQUEST:
            {
                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), "");

                // Create Reply
                Signal signal = Signal(STOP_GAME_REPLY, sizeof(StopGameReply));
                signal.getSignal()->stopGameReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->stopGameReply.result));
                *new_sock << signal;

                break;
            }

            case DEREGISTER_PLAYER_REQUEST:
            {
                TRACE_REC_SIG(clientName.c_str(), serverName.c_str(), data.getSignalNumber(), STR("player %s", data.getSignal()->deregisterPlayerRequest.player));

                // Create Reply
                Signal signal = Signal(DEREGISTER_PLAYER_REPLY, sizeof(DeregisterPlayerReply));
                signal.getSignal()->deregisterPlayerReply.result = true;

                TRACE_SEND_SIG(serverName.c_str(), clientName.c_str(), signal.getSignalNumber(), STR("result %d", signal.getSignal()->deregisterPlayerReply.result));
                *new_sock << signal;

                isFinished = true;
                break;
            }
            default:
                TRACE_ERROR(STR("Unknown signal %d", data.getSignalNumber()));
                break;
        }
    }    

    RETURN;
}

void sigchld_handler(int s)
{
    while(waitpid(-1, NULL, WNOHANG) > 0);
}

// get sockaddr, IPv4 or IPv6:
void *get_in_addr(struct sockaddr *sa)
{
    if (sa->sa_family == AF_INET) {
        return &(((struct sockaddr_in*)sa)->sin_addr);
    }

    return &(((struct sockaddr_in6*)sa)->sin6_addr);
}

